package com.iver.andami.ui.splash;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileFilter;
import java.net.MalformedURLException;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;

import com.iver.andami.messages.Messages;

public class MultiSplashWindow extends JWindow implements TimerCallBack {

	private static final long serialVersionUID = 572592158521767258L;

	private Timer timer;
	private int numLogos = 0;
	private ImageIcon[] img = null;
	private static JProgressBar pb;
	private static JLabel lblStatus;
	private Dimension splashDimension;
	private int index = 0;
	private int current;
	private int lastIndex=-1;

	public MultiSplashWindow(Frame f) {
		super(f);

		lblStatus = new JLabel(Messages.getString("SplashWindow.Iniciando"));
		lblStatus.setBorder(BorderFactory.createEtchedBorder());
		lblStatus.setBackground(Color.WHITE);

		pack();
		pb = new JProgressBar();
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(lblStatus, BorderLayout.NORTH);
		getContentPane().add(pb, BorderLayout.SOUTH);

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		init();
		setLocation((screenSize.width / 2) - (splashDimension.width / 2),
				(screenSize.height / 2) - (splashDimension.height / 2));
		index = 0;
		setVisible(true);

	}

	public void init(){
   	 File dirLogos=new File("./logos");
   	 File[] fileLogos=dirLogos.listFiles(new FileFilter() {

			public boolean accept(File arg0) {
				if (arg0.isDirectory())
					return false;
				if (!(arg0.getName().toUpperCase().endsWith(".PNG")||
						arg0.getName().toUpperCase().endsWith(".BMP")||
						arg0.getName().toUpperCase().endsWith(".GIF")||
						arg0.getName().toUpperCase().endsWith(".JPG")||
						arg0.getName().toUpperCase().endsWith(".JPEG")))
					return false;
				return true;
			}});
   	 if (fileLogos!=null) {
   		numLogos=fileLogos.length+1;
   	 }else {
   		numLogos=1;
   	 }

   	 img=new ImageIcon[numLogos];
   	 img[0]=new ImageIcon(this.getClass().getClassLoader().getResource("images/logo_es.png"));
   	 int splashWidth = img[0].getIconWidth();
   	 int splashHeight = img[0].getIconHeight();

   	 for (int i=0;i<numLogos-1;i++){
   		 try {
				img[i+1]=new ImageIcon(fileLogos[i].toURL());
				splashWidth = Math.max(splashWidth, img[i+1].getIconWidth());
				splashHeight = Math.max(splashHeight, img[i+1].getIconHeight());
   		 } catch (MalformedURLException e) {
		}
   	 }
   	 splashDimension = new Dimension(splashWidth, splashHeight);
   	 setSize(splashDimension.width, splashDimension.height + 20);

   	 start();
    }

	public void paint(Graphics g) {
		if (lastIndex==current)
			return;
		super.paint(g);

		if (img == null || img[current] == null)
			return;

		g.drawImage(img[current].getImage(), 0, 20, this);
		lastIndex=current;
	}

	public void tick() {
		current = index;
		repaint();
		index++;
		if (index >= numLogos)
			index = 0;
	}

	public void start() {
		timer = new Timer(this, 1000);
		timer.start();
	}

	/**
	 * cierra la ventana
	 */
	public void close() {
		setVisible(false);
		stop();
		dispose();
	}

	public void stop() {
		timer.stop();
		timer = null;
	}

	public static void main(String[] args) {
		Frame frame = new Frame();
		MultiSplashWindow ba = new MultiSplashWindow(frame);
		ba.setVisible(true);
		ba.setSize(500, 500);
		ba.init();
		ba.start();
	}

	/**
	 * @see com.iver.mdiApp.ui.AppLoadingListener#process(int)
	 */
	public static void process(int p, String str) {

		lblStatus.setText(str);
		// System.out.println("Porcentaje :" + p);
		if (pb.getValue() != p) {
			pb.setValue(p);
			pb.repaint();
			// pb.paint(pb.getGraphics());
		}
	}
}
