
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.catalogClient.schemas.parsers;
import java.net.URL;

import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.metadataXML.XMLTree;
import es.gva.cit.catalogClient.querys.Coordinates;
import es.gva.cit.catalogClient.schemas.discoverer.Record;
import es.gva.cit.catalogClient.schemas.discoverer.Resource;

/**
 * 
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class GeonetworkISO19115Parser extends Record {

/**
 * Constructor
 * 
 * 
 * @param node Node with the answer
 * @param serverURL Server URL. Necessary to load the image (just Geonetwork)
 */
    public  GeonetworkISO19115Parser(XMLNode node, URL serverURL) {        
        setNode(node);
        setTitle(XMLTree.searchNodeValue(node,
                "dataIdInfo->idCitation->resTitle"));
        setAbstract_(XMLTree.searchNodeValue(node, "dataIdInfo->idAbs"));
        setPurpose(XMLTree.searchNodeValue(node, "dataIdInfo->idPurp"));
        setKeyWords(XMLTree.searchMultipleNodeValue(node,
                "dataIdInfo->descKeys->keyword"));
        setResources(getResources("distInfo->distTranOps->onLineSrc"));
        setFileID(XMLTree.searchNodeValue(node,"mdFileID"));
        setServerURL(serverURL);
        //Caution: getImageUrl uses serverURL and FileID!!!
        setImage(getImageUrl("dataIdInfo->graphOver"));
        
    } 

/**
 * 
 * 
 * 
 * @return 
 * @param label 
 */
    private Resource[] getResources(String label) {        
        XMLNode[] nodes = XMLTree.searchMultipleNode(getNode(), label);
        Coordinates coordinates = null;
        String srs = null;
        
        if (nodes == null) {
            return null;
        }
        Resource[] resources = new Resource[nodes.length];
        if (nodes.length > 0){
        	srs = XMLTree.searchNodeValue(getNode(),"refSysInfo->MdCoRefSys->refSysID->identCode");
        	coordinates = new Coordinates(XMLTree.searchNodeValue(getNode(),"dataIdInfo->geoBox->westBL"),
        			XMLTree.searchNodeValue(getNode(),"dataIdInfo->geoBox->northBL"),
					XMLTree.searchNodeValue(getNode(),"dataIdInfo->geoBox->eastBL"),
					XMLTree.searchNodeValue(getNode(),"dataIdInfo->geoBox->southBL"));
        }
        	
        	
        for (int i = 0; i < resources.length; i++){
            	resources[i] = new Resource(XMLTree.searchNodeValue(nodes[i],
                        "linkage"),
                    XMLTree.searchNodeValue(nodes[i], "protocol"),
                    XMLTree.searchNodeValue(nodes[i], "orName"),
                    XMLTree.searchNodeValue(nodes[i], "orDesc"),
                    XMLTree.searchNodeAtribute(nodes[i], "orFunct->OnFunctCd",
                        "value"),
					srs,	
            		coordinates);
            	if (resources[i].getLinkage() == null){
            		resources[i].setLinkage("");
            	}
            	
        }
        
        
        return resources;
    }     
  
    
 }
