
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gazetteer.adl.drivers;
import es.gva.cit.catalogClient.metadataXML.XMLNode;
import es.gva.cit.catalogClient.protocols.HTTPPostProtocol;
import es.gva.cit.gazetteer.adl.filters.ADLFilter;
import es.gva.cit.gazetteer.adl.parsers.AdlCapabilitiesParser;
import es.gva.cit.gazetteer.adl.parsers.AdlFeatureParser;
import es.gva.cit.gazetteer.drivers.AsbtractGazetteerServiceDriver;
import es.gva.cit.gazetteer.querys.Feature;
import es.gva.cit.gazetteer.querys.Query;
import es.gva.cit.gazetteer.querys.ThesaurusName;
import java.net.URL;
import java.util.Collection;

/**
 * This class implements the driver to connect with a ADL gazetteer service
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 * @see http://alexandria.sdc.ucsb.edu/
 */
public class ADLGazetteerServiceDriver extends AsbtractGazetteerServiceDriver {

/**
 * It invokes the getCapabilities operation
 * 
 * 
 * @return 
 * @param url 
 */
    public Collection getCapabilities(URL url) {        
       Collection nodes = new java.util.ArrayList();
       System.out.println("**************POST*************");
       nodes = new HTTPPostProtocol().doQuery(url,
               getPOSTMessageCapabilities(), 0);
        
       return nodes;
    } 

/**
 * It creates the XML for the getCapabilities request
 * 
 * 
 * @return Name-value pair with a XML request
 */
    private String getPOSTMessageCapabilities() {        
        return "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" +
        	"<gazetteer-service " +
        	"xmlns=\"http://www.alexandria.ucsb.edu/gazetteer\" " +
        	"version=\"1.2\">" +
        	"<get-capabilities-request/>" +
        	"</gazetteer-service>";
        
    } 

/**
 * It invokes the getFeature operation
 * 
 * 
 * @return 
 * @param url 
 * @param query 
 */
    public Collection getFeature(URL url, Query query) {        
        Collection nodes = new java.util.ArrayList();
        setQuery(query);
              
        
      System.out.println("**************POST*************");
        System.out.println(getPOSTGetFeature(getQuery(),true));
        nodes = new HTTPPostProtocol().doQuery(url,
                       getPOSTGetFeature(getQuery(),true), 0);
           
       if (nodes == null){
    	   System.out.println(getPOSTGetFeature(getQuery(),false));
    	   	nodes = new HTTPPostProtocol().doQuery(url,
                getPOSTGetFeature(getQuery(),false), 0);
        }
        return nodes;
        
    } 

/**
 * It creates the XML for the getFeature request
 * 
 * 
 * @return Name-value pair with a XML request
 * @param query 
 */
    private String getPOSTGetFeature(Query query,boolean withAccents) {        
               
        return  "<gazetteer-service " +
                "xmlns=\"http://www.alexandria.ucsb.edu/gazetteer\" " +
                "xmlns:gml=\"http://www.opengis.net/gml\" " +
                "version=\"1.2\">" +
                		new ADLFilter(withAccents).getQuery(query) +
                "</gazetteer-service>" ;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#isProtocolSupported(java.net.URL)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 */
    public boolean isProtocolSupported(URL url) {        
        // TODO Auto-generated method stub
        return true;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#setParameters(org.w3c.dom.Node[])
     */

/**
 * 
 * 
 * 
 * @return 
 * @param url 
 * @param feature 
 */
    public Collection describeFeatureType(URL url, String feature) {        
        // TODO Auto-generated method stub
        return null;
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#setParameters(org.w3c.dom.Node[])
     */

/**
 * 
 * 
 * 
 * @return 
 * @param nodes 
 */
    public boolean setParameters(Collection nodes) {        
        return new AdlCapabilitiesParser(this).parse((XMLNode)nodes.toArray()[0]);
    } 
/* (non-Javadoc)
     * @see es.gva.cit.gazetteer.drivers.IGazetteerServiceDriver#parseFeatures(org.w3c.dom.Node)
     */

/**
 * 
 * 
 * 
 * @return 
 * @param node 
 * @param currentFeature 
 * @param attribute 
 */
    public Feature[] parseFeatures(XMLNode node, ThesaurusName currentFeature, String attribute) {        
        return new AdlFeatureParser().parse(node);
    } 
 }
