
/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
*
* Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
*
* For more information, contact:
*
*  Generalitat Valenciana
*   Conselleria d'Infraestructures i Transport
*   Av. Blasco Ibez, 50
*   46010 VALENCIA
*   SPAIN
*
*      +34 963862235
*   gvsig@gva.es
*      www.gvsig.gva.es
*
*    or
*
*   IVER T.I. S.A
*   Salamanca 50
*   46005 Valencia
*   Spain
*
*   +34 963163400
*   dac@iver.es
*/
package es.gva.cit.gazetteer.ui.serverConnect;
import com.iver.utiles.swing.jcomboServer.ServerData;
import es.gva.cit.gazetteer.GazetteerClient;
import es.gva.cit.gazetteer.ui.search.SearchDialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Iterator;
import java.util.TreeMap;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

import org.gvsig.i18n.Messages;

/**
 * 
 * 
 * 
 * @author Jorge Piera Llodra (piera_jor@gva.es)
 */
public class ServerConnectDialogPanel extends JPanel implements ActionListener {

/**
 * 
 * 
 */
    private static TreeMap serverList = new TreeMap();
//Panels
/**
 * 
 * 
 */
    private JPanel ppalPanel = null;
/**
 * 
 * 
 */
    private ServerConnectPanel controlsPanel = null;
/**
 * 
 * 
 */
    private JPanel buttonsPanel = null;
//Buttons
/**
 * 
 * 
 */
    private JButton connectButton = null;
/**
 * 
 * 
 */
    private JButton searchButton = null;
/**
 * 
 * 
 */
    private JButton closeButton = null;
/**
 * 
 * 
 */
    private JFrame parent = null;
//Others
/**
 * 
 * 
 */
    protected GazetteerClient client = null;

/**
 * 
 * 
 */
    protected String serversFile = "servers/GazetteerServers.txt";

/**
 * 
 * 
 */
    protected String currentServer = "";

/**
 * 
 * 
 * 
 * @param parent 
 * @param translator 
 */
    public  ServerConnectDialogPanel(JFrame parent) {
        this.parent = parent;
        ppalPanel = new JPanel();
        ppalPanel.setPreferredSize(new Dimension(600,265));
        ppalPanel.setLayout(new BoxLayout(ppalPanel, BoxLayout.Y_AXIS));
        ppalPanel.add(getControlsPanel(), null);
        ppalPanel.add(getButtonPanel(), null);
        add(ppalPanel);
        setDefaultButtonListeners();
        //Loads the servers
        loadServerList(serversFile);
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public JPanel getControlsPanel() {        
        if (controlsPanel == null) {
            controlsPanel = new ServerConnectPanel();
            controlsPanel.getWFSGButton().addActionListener(this);
            controlsPanel.getADLButton().addActionListener(this);
            controlsPanel.getServidoresCombo().addActionListener(this);
        }
        return controlsPanel;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public JPanel getButtonPanel() {        
        if (buttonsPanel == null) {
        	FlowLayout flowLayout = new FlowLayout();
        	flowLayout.setAlignment(FlowLayout.RIGHT);
            buttonsPanel = new JPanel(flowLayout);
            buttonsPanel.add(getConnectButton());
            buttonsPanel.add(getSearchButton());
            buttonsPanel.add(getCloseButton());
        }
        return buttonsPanel;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public JButton getConnectButton() {        
        if (connectButton == null) {
            connectButton = new JButton(Messages.getText("connectButton"));
            connectButton.setPreferredSize(new Dimension(90, 23));
            connectButton.setActionCommand("Connect");
        }
        return connectButton;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public JButton getSearchButton() {        
        if (searchButton == null) {
            searchButton = new JButton(Messages.getText("searchButton"));
            searchButton.setPreferredSize(new Dimension(80, 23));
            searchButton.setActionCommand("Search");
            searchButton.setEnabled(false);
        }
        return searchButton;
    } 

/**
 * 
 * 
 * 
 * @return 
 */
    public JButton getCloseButton() {        
            if (closeButton == null) {
                closeButton = new JButton(Messages.getText("close"));
                closeButton.setPreferredSize(new Dimension(80, 23));
                closeButton.setActionCommand("close");
               
            }
            return closeButton;
    } 

/**
 * It adds a server in the TreeMap Object
 * 
 * 
 * @param server 
 */
    protected static void addTreeMapServer(ServerData server) {        
        if (ServerConnectDialogPanel.serverList == null) {
            ServerConnectDialogPanel.serverList = new TreeMap();
        }
        serverList.put(server.getServerAddress(), server);
      
    } 

/**
 * Sets the listeners
 * 
 */
    public void setDefaultButtonListeners() {        
        getConnectButton().addActionListener(this);
        getSearchButton().addActionListener(this);
        getCloseButton().addActionListener(this);
    } 

/**
 * This method loads a server list in the combo
 * 
 * 
 * @param sfile 
 */
    private void loadServerList(String sfile) {        
        loadServersFromFile(sfile);
        
        Iterator iter = serverList.keySet().iterator();
        while (iter.hasNext()) {
            ServerData server = (ServerData) serverList.get((String) iter.next());
            controlsPanel.getServerCombo().addServer(server);
        }            
    } 

/**
 * It loads a server list from a text file
 * 
 * 
 * @param sfile 
 */
    private void loadServersFromFile(String sfile) {        
        File file = null;
        try {
            file = new File(sfile);
            if (file.exists()) {
                BufferedReader fr = new BufferedReader(new FileReader(file));
                String s;
                while ((s = fr.readLine()) != null) {
                    addTreeMapServer(new ServerData(s,"",""));
                }
            } else {
                System.out.println("No se encuentra el fichero '" +
                    file.getPath() + "'");
            }
        } catch (FileNotFoundException e) {
            System.out.println("No se encuentra el fichero '" + file.getPath() +
                "'");
            //e.printStackTrace();
        } catch (IOException e) {
            System.out.println("Error de entrada salida en la lectura del fichero");
            //e.printStackTrace();
        }
    } 
/* (non-Javadoc)
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */

/**
 * 
 * 
 * 
 * @param e 
 */
    public void actionPerformed(ActionEvent e) {        
        //Connect
        if (e.getActionCommand() == "Connect") {
           connectButtonActionPerformed();
        }
        //BUSCAR
        if (e.getActionCommand() == "Search") {
            searchButtonActionPerformed();
        }
        if ((e.getActionCommand() == "WFS-G") ||
                (e.getActionCommand() == "ADL") ||
                (e.getActionCommand() == "comboBoxChanged")) {
            searchButton.setEnabled(false);
        }
        
        if (e.getActionCommand() == "servidoresCombo"){
            try {
                controlsPanel.setProtocol(controlsPanel.getServer().getServiceSubType());
            }catch(NullPointerException ex){
                //The server is not loaded 
            }
            
        }
        //Connect
        if (e.getActionCommand() == "close") {
           closeButtonActionPerformed();
        }
              
    } 

/**
 * 
 * 
 */
    public void searchButtonActionPerformed() {        
        setEnabled(false);
        new SearchDialog(client,parent);
    } 

/**
 * 
 * 
 */
    public void connectButtonActionPerformed() {        
        searchButton.setEnabled(false);
//      Create a new Gazetteer client
        client = new GazetteerClient(controlsPanel.getServerAddress(),
                controlsPanel.getProtocol());
        
        if (client.getCapabilities()){
            searchButton.setEnabled(true);
            currentServer = controlsPanel.getServerAddress();
            searchButtonActionPerformed();
        }
        
        controlsPanel.setRespuesta(Messages.getText(client.getServerStatus()));
    } 

/**
 * 
 * 
 */
    public void closeButtonActionPerformed() {        
        parent.setVisible(false);
    } 

/**
 * 
 * 
 * 
 * @return Returns the serversFile.
 */
    public String getServersFile() {        
        return serversFile;
    } 

/**
 * 
 * 
 * 
 * @param serversFile The serversFile to set.
 */
    public void setServersFile(String serversFile) {        
        this.serversFile = serversFile;
    } 

/**
 * 
 * 
 * 
 * @return Returns the currentServer.
 */
    public String getCurrentServer() {        
        return currentServer;
    } 

/**
 * 
 * 
 * 
 * @return Returns the client.
 */
    public GazetteerClient getCliente() {        
        return client;
    } 
 }
