/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig;

import java.awt.Component;
import java.io.IOException;

import javax.swing.JOptionPane;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.instruction.EvaluationException;
import com.hardcode.gdbms.engine.instruction.SemanticException;
import com.hardcode.gdbms.parser.ParseException;
import com.iver.andami.PluginServices;
import com.iver.andami.messages.NotificationManager;
import com.iver.andami.plugins.Extension;
import com.iver.andami.ui.mdiManager.IWindow;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.LayerFactory;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.layerOperations.AlphanumericData;
import com.iver.cit.gvsig.gui.filter.DefaultExpressionDataSource;
import com.iver.cit.gvsig.gui.filter.ExpressionListener;
import com.iver.cit.gvsig.gui.filter.FilterDialog;
import com.iver.cit.gvsig.project.documents.table.gui.Table;
import com.iver.cit.gvsig.project.documents.view.IProjectView;
import com.iver.utiles.exceptionHandling.ExceptionListener;


/**
 * Extensin que abre un dilogo para poder hacer un filtro de una capa o tabla.
 *
 * @author Vicente Caballero Navarro
 */
public class FiltroExtension extends Extension implements ExpressionListener {
	protected SelectableDataSource dataSource = null;
	protected Table vista;
	private String filterTitle;

	/**
	 * DOCUMENT ME!
	 */
	public void initialize() {
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param actionCommand DOCUMENT ME!
	 */
	public void execute(String actionCommand) {
		if ("FILTRO".equals(actionCommand)) {
			try {
				IWindow v = PluginServices.getMDIManager().getActiveWindow();

				if (v instanceof Table) {
					vista = (Table) v;

					dataSource = (SelectableDataSource)vista.getModel().getModelo().getRecordset();
					filterTitle = vista.getModel().getName();

				} else if (v instanceof com.iver.cit.gvsig.project.documents.view.gui.View) {
					IProjectView pv = ((com.iver.cit.gvsig.project.documents.view.gui.View) v).getModel();
					filterTitle = ((com.iver.cit.gvsig.project.documents.view.gui.View) v).getModel().getName();
					FLayer layer = pv.getMapContext()
					.getLayers().getActives()[0];
					dataSource = pv.getProject().getDataSourceByLayer(layer);
				}
			}  catch (DriverException e) {
				NotificationManager.addError("Error filtrando", e);
			}

			doExecute();
		}
	}

	/**
	 * "execute" method action.
	 *
	 */
	protected void doExecute(){
		try{
			DefaultExpressionDataSource ds = new DefaultExpressionDataSource();
			ds.setTable(dataSource);

			FilterDialog dlg = new FilterDialog(filterTitle);
			dlg.addExpressionListener(this);
			dlg.addExceptionListener(new ExceptionListener() {
				public void exceptionThrown(Throwable t) {
					NotificationManager.addError(t.getMessage(), t);
				}
			});

			dlg.setModel(ds);
			PluginServices.getMDIManager().addWindow(dlg);
		} catch (DriverLoadException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean isEnabled() {
		IWindow v = PluginServices.getMDIManager().getActiveWindow();

		if (v == null) {
			return false;
		}

		if (v instanceof Table) {
			return true;
		} else {
			if (v instanceof com.iver.cit.gvsig.project.documents.view.gui.View) {
				com.iver.cit.gvsig.project.documents.view.gui.View view = (com.iver.cit.gvsig.project.documents.view.gui.View) v;
				IProjectView pv = view.getModel();
				FLayer[] seleccionadas = pv.getMapContext().getLayers()
				.getActives();

				if (seleccionadas.length == 1) {
					if (seleccionadas[0].isAvailable() && seleccionadas[0] instanceof AlphanumericData) {
						return true;
					}
				}
			}

			return false;
		}

	}

	/**
	 * DOCUMENT ME!
	 *
	 * @return DOCUMENT ME!
	 */
	public boolean isVisible() {
		IWindow v = PluginServices.getMDIManager().getActiveWindow();

		if (v == null) {
			return false;
		}

		if (v instanceof Table) {
			return true;
		} else {
			if (v instanceof com.iver.cit.gvsig.project.documents.view.gui.View) {
				com.iver.cit.gvsig.project.documents.view.gui.View view = (com.iver.cit.gvsig.project.documents.view.gui.View) v;
				IProjectView pv = view.getModel();
				FLayer[] seleccionadas = pv.getMapContext().getLayers()
				.getActives();

				if (seleccionadas.length == 1) {
					if (seleccionadas[0] instanceof AlphanumericData) {
						return true;
					}
				}
			}

			return false;
		}
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expression DOCUMENT ME!
	 */
	public void newSet(String expression) {
		
		try {
			long[] sel = doSet(expression);
	
			if (sel == null) {
				//throw new RuntimeException("Not a 'where' clause?");
				return;
			}
	
			FBitSet selection = new FBitSet();
	
			for (int i = 0; i < sel.length; i++) {
				selection.set((int) sel[i]);
			}
	
			dataSource.clearSelection();
			dataSource.setSelection(selection);
		}catch(Exception e){
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(), "Asegurate de que la consulta es correcta.");
		}
		
	}

	/**
	 * @see com.iver.cit.gvsig.gui.filter.ExpressionListener#newSet(java.lang.String)
	 */
	private long[] doSet(String expression) {
		try {
			DataSource ds = LayerFactory.getDataSourceFactory().executeSQL(expression,
					DataSourceFactory.MANUAL_OPENING);

			return ds.getWhereFilter();
		} catch (DriverLoadException e) {
			NotificationManager.addError("Error cargando el driver", e);
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			NotificationManager.addError("Error accediendo al driver", e);
		} catch (ParseException e) {
			NotificationManager.addError("Parse error", e);
		} catch (SemanticException e) {
			NotificationManager.addError(e.getMessage(), e);
		} catch (IOException e) {
			NotificationManager.addError("GDBMS internal error", e);
		} catch (EvaluationException e) {
			NotificationManager.addError("Sintaxis incorrecta", e);
		}

		return null;
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expression DOCUMENT ME!
	 */
	public void addToSet(String expression) {
		long[] sel = doSet(expression);

		if (sel == null) {
			//throw new RuntimeException("Not a 'where' clause?");
			return;
		}

		FBitSet selection = new FBitSet();

		for (int i = 0; i < sel.length; i++) {
			selection.set((int) sel[i]);
		}

		FBitSet fbs = dataSource.getSelection();
		fbs.or(selection);
		dataSource.setSelection(fbs);
	}

	/**
	 * DOCUMENT ME!
	 *
	 * @param expression DOCUMENT ME!
	 */
	public void fromSet(String expression) {
		long[] sel = doSet(expression);

		if (sel == null) {
			throw new RuntimeException("Not a 'where' clause?");
		}

		FBitSet selection = new FBitSet();

		for (int i = 0; i < sel.length; i++) {
			selection.set((int) sel[i]);
		}

		FBitSet fbs = dataSource.getSelection();
		fbs.and(selection);
		dataSource.setSelection(fbs);
	}
}
