package es.prodevelop.cit.gvsig.arcims.fmap.drivers;

import java.awt.geom.Rectangle2D;
import java.io.IOException;

import org.gvsig.remoteClient.arcims.exceptions.ArcImsException;

import com.hardcode.driverManager.DriverLoadException;
import com.hardcode.gdbms.engine.data.DataSource;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.drivers.DriverIOException;
import com.iver.cit.gvsig.fmap.layers.FBitSet;
import com.iver.cit.gvsig.fmap.layers.SelectableDataSource;
import com.iver.cit.gvsig.fmap.layers.VectorialAdapter;

public class ArcImsVectorialAdapter extends VectorialAdapter {

	private IFeature[] testArray;

	public ArcImsVectorialAdapter(FMapFeatureArcImsDriver drv) {
		setDriver(drv);
	}

	public IGeometry getShape(int index) throws DriverIOException {
		try {
			return getDriver().getShape(index);
		} catch (IOException e) {
			DriverIOException dioe = new DriverIOException(e.getMessage());
			throw dioe;
		}
	}

	public int getShapeCount() throws DriverIOException {
		try {
			return getDriver().getShapeCount();
		} catch (IOException e) {
			DriverIOException dioe = new DriverIOException(e.getMessage());
			throw dioe;
		}
	}

	public Rectangle2D getFullExtent() throws DriverIOException {
		try {
			return getDriver().getFullExtent();
		} catch (IOException e) {
			DriverIOException dioe = new DriverIOException(e.getMessage());
			throw dioe;
		}
	}

	public int getShapeType() throws DriverIOException {
		return getDriver().getShapeType();
	}

	/**
	 * This method is used when the data source is a proper database that needs
	 * initialization.
	 */
	public void start() throws DriverIOException {
		try {
			((FMapFeatureArcImsDriver) getDriver()).getRecordSet().start();
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			DriverIOException dioe = new DriverIOException(e.getMessage());
			throw dioe;
		}
	}

	/**
	 * This method is used when the data source is a proper database that needs
	 * to be closed after use.
	 */
	public void stop() throws DriverIOException {
		try {
			((FMapFeatureArcImsDriver) driver).getRecordSet().stop();
		} catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
			DriverIOException dioe = new DriverIOException(e.getMessage());
			throw dioe;
		}
	}

	public SelectableDataSource getRecordset() throws DriverLoadException {
		return ((FMapFeatureArcImsDriver) driver).getRecordSet();
	}

	public void requestFeatureAttributes(FBitSet fbs) throws ArcImsException {
		((FMapFeatureArcImsDriver) driver).requestFeatureAttributes(fbs);
		}
	

}

// public class ArcImsVectorialAdapter implements ReadableVectorial {
//
// private FMapFeatureArcImsDriver driver;
// private IGeometry[] testArray;
// private Random rnd = new Random(System.currentTimeMillis());
// private DataSource dataSource;
//
// public ArcImsVectorialAdapter() {
//	
// testArray = new IGeometry[10];
// for (int i=0; i<testArray.length; i++) {
// Rectangle2D.Double r = getRandomRect();
// GeneralPathX gp = new GeneralPathX(r);
// testArray[i] = ShapeFactory.createPolygon2D(gp);
// }
//	
// }
//
// private Double getRandomRect() {
// double x = -180.0 * rnd.nextDouble();
// double y = -90.0 * rnd.nextDouble();
// double w = 180.0 * rnd.nextDouble();
// double h = 90.0 * rnd.nextDouble();
// return new Rectangle2D.Double(x, y, w, h);
// }
//
// /**
// * This method is used when the data source is a proper database
// * that needs initialization.
// */
// public void start() throws DriverIOException {
// System.out.println("===> public void start()");
// // TODO Auto-generated method stub
// }
//
// /**
// * This method is used when the data source is a proper database
// * that needs to be closed after use.
// */
// public void stop() throws DriverIOException {
// System.out.println("===> public void stop()");
// // TODO Auto-generated method stub
// }
//
// public IGeometry getShape(int index) throws DriverIOException {
// // TODO Auto-generated method stub
// return testArray[index];
// }
//
// public int getShapeCount() throws DriverIOException {
// // TODO Auto-generated method stub
// return testArray.length;
// }
//
// public Rectangle2D getFullExtent() throws DriverIOException {
// // TODO Auto-generated method stub
// System.out.println("===> public Rectangle2D getFullExtent()");
// return null;
// }
//
// public int getShapeType() throws DriverIOException {
// // TODO Auto-generated method stub
// System.out.println("===> public int getShapeType()");
// return 0;
// }
//
// public FMapFeatureArcImsDriver getDriver() {
// return driver;
// }
//
// public void setDriver(FMapFeatureArcImsDriver driver) {
// this.driver = driver;
// }
//
// public Rectangle2D getTestExtent() {
// return new Rectangle(-180, -90, 360, 180);
// }
//
// public DataSource getRecordset() throws DriverLoadException {
//	
// if (dataSource != null) return dataSource;
//	
// if (getDriver() instanceof ObjectDriver)
// {
// String name =
// LayerFactory.getDataSourceFactory().addDataSource((ObjectDriver)
// getDriver());
// try {
// dataSource = LayerFactory.getDataSourceFactory().createRandomDataSource(name,
// DataSourceFactory.AUTOMATIC_OPENING);
// return dataSource;
// } catch (NoSuchTableException e) {
// throw new RuntimeException(e);
// } catch (com.hardcode.gdbms.engine.data.driver.DriverException e) {
// throw new RuntimeException(e);
// }
// }
// System.err.println("Unable to get Data source in method public DataSource
// getRecordset().");
// System.err.println("class: ArcImsVectorialAdapter.");
// return null;
// }
//
// public void setRecordSet(DataSource ds) {
// dataSource = ds;
// }
//
// }

