/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.dialog;

import java.awt.FlowLayout;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;

import com.iver.andami.PluginServices;

/**
 * Control para el manejo de tablas. No contiene eventos, estos deben 
 * ser manejados desde la clase que lo llame.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class DialogButtons extends JPanel{
	
	//**********************Vars**********************************		
	private JButton 		bSave = null;
	private JButton 		bLoad = null;
	private JButton 		bExportToCSV = null;
	private JButton 		bImportFromCSV = null;
	private String			pathToImages = "images/";//"/com/iver/cit/gvsig/gui/panels/images/";
	//**********************End Vars******************************
	
	//**********************Methods*******************************
	/**
	 * This is the default constructor
	 */
	public DialogButtons() {
		super();
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {                
		this.setSize(new java.awt.Dimension(91,22));
		FlowLayout flowLayout5 = new FlowLayout();
		flowLayout5.setHgap(0);
		flowLayout5.setVgap(0);
        
		this.setLayout(flowLayout5);

		this.add(getBSave(), null);
		this.add(getBLoad(), null);
		this.add(getBExportToCSV(), null);
		this.add(getBImportFromCSV(), null);
	}
	//**********************End Methods***************************	
	
	//**********************Getters & Setters*********************
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBImportFromCSV() {
		if (bImportFromCSV == null) {
			bImportFromCSV = new JButton();
			bImportFromCSV.setText("");
			bImportFromCSV.setPreferredSize(new java.awt.Dimension(22,22));
			try{
				bImportFromCSV.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "importFromCSV.png")));
			}catch(NullPointerException e){
				//Sin icono
			}
			bImportFromCSV.setEnabled(true);
			bImportFromCSV.setSelected(true);
			bImportFromCSV.setToolTipText(PluginServices.getText(this, "import_from_ascii"));
		}
		return bImportFromCSV;
	}
	
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBLoad() {
		if (bLoad == null) {
			bLoad = new JButton();
			bLoad.setPreferredSize(new java.awt.Dimension(22,22));
			bLoad.setEnabled(true);
			try{
				bLoad.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "load.png")));
			}catch(NullPointerException e){
				//Sin icono
			}
			bLoad.setToolTipText(PluginServices.getText(this, "load_from_xml"));
		}
		return bLoad;
	}
	
	/**
	 * This method initializes bSave	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public JButton getBSave() {
		if (bSave == null) {
			bSave = new JButton();
			bSave.setText("");
			bSave.setEnabled(false);
			bSave.setPreferredSize(new java.awt.Dimension(22,22));
			try{
				bSave.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "save.png")));
			}catch(NullPointerException e){
				//Sin icono
			}
			bSave.setActionCommand("");
			bSave.setToolTipText(PluginServices.getText(this, "save_to_xml"));
		}
		return bSave;
	}
	
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */    
	public JButton getBExportToCSV() { 
		if (bExportToCSV == null) {
			bExportToCSV = new JButton();
			bExportToCSV.setPreferredSize(new java.awt.Dimension(22,22));
			bExportToCSV.setEnabled(false);
			try{
				bExportToCSV.setIcon(new ImageIcon(getClass().getClassLoader().getResource(pathToImages + "exportToCSV.png")));
			}catch(NullPointerException e){
				//Sin icono
			}
			bExportToCSV.setToolTipText(PluginServices.getText(this, "save_to_ascii"));
		}
		return bExportToCSV;
	}
	
	/**
	 * Activa o desactiva los botones de exportar capa de puntos y salvar
	 * la tabla a Ascii.
	 *@param isPossible Si vale true activa los controles de salvar y exportacin
	 *y si vale false los desactiva
	 */
	public void setSaveToDisk(boolean isPossible){
		this.getBSave().setEnabled(isPossible);
		this.getBExportToCSV().setEnabled(isPossible);
	}
	//**********************End Getters & Setters*****************
 }
