/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.listeners;

import java.awt.Component;
import java.awt.geom.Point2D;

import javax.swing.JOptionPane;

import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.FLayers;
import com.iver.cit.gvsig.fmap.layers.FLyrGeoRaster;
import com.iver.cit.gvsig.fmap.layers.FLyrPoints;
import com.iver.cit.gvsig.fmap.tools.BehaviorException;
import com.iver.cit.gvsig.fmap.tools.GeorefDraggListenerImpl;
import com.iver.cit.gvsig.fmap.tools.Events.PointEvent;
import com.iver.cit.gvsig.project.documents.view.gui.View;


/**
 * <p>Extensin de la clase GeorefPointerListenerImpl de FMap para poder llamar a mtodos de
 * andami o de gvSIG.</p>
 * <p>Este listener mueve puntos desde una vista correspondientes a la capa de puntos.</p>
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class GeorefMovePointListener extends GeorefDraggListenerImpl {
	
	private GeoreferencingDialog dialog = null;
	private FLyrPoints lyrPoints = null;
	private FLyrGeoRaster lyrGeoRaster = null;
	private View theView = null;
	private ViewPort viewPort = null;
	
	/**
	 * Posicin del punto a ser seleccionado. 
	 */
	private int posPoint = -1;
	/**
	 * Es true si el punto que se ha movido corresponde con el pixel 
	 * de la imagen y false si corresponde con el punto de la vista
	 */
	private boolean isPixel = true;
	
	/**
	 * Constructor
	 *
	 * @param mapCtrl
	 */
	public GeorefMovePointListener(GeoreferencingDialog dialog) {
		super();
		this.dialog = dialog;
	}

	/**
	 * Captura de puntos sobre una capa FlyrPoint
	 *
	 * @param event Evento de un punto sobre la vista
	 *
	 * @throws BehaviorException
	 */
	public void press(PointEvent event) throws BehaviorException {
		super.press(event);

		try{
			theView = (View)PluginServices.getMDIManager().getActiveWindow();
		}catch(ClassCastException exc){
			return;
		}
		viewPort = theView.getMapControl().getMapContext().getViewPort();
		FLayers flayers = theView.getMapControl().getMapContext().getLayers();
		
		//Obtencin de la capa de puntos.

		for(int i=0;i<flayers.getLayersCount();i++){
   		 FLayer lyr = flayers.getLayer(i);
   		 if(lyr instanceof FLyrGeoRaster)
   			 lyrGeoRaster = (FLyrGeoRaster)lyr;
   		}
		lyrPoints = lyrGeoRaster.getFLyrPoints();
   		if(lyrGeoRaster == null || lyrPoints == null)
   			return;
   		 
		
		//Hallamos el punto ms cercano a donde hemos pinchado
		Point2D init = viewPort.toMapPoint(event.getPoint());
		double distance = Double.MAX_VALUE;
		
		for(int i=0;i<lyrPoints.length();i++){
			Point2D pixel = lyrGeoRaster.img2World(lyrPoints.getPoint(i).pixelPoint);
			Point2D map = lyrPoints.getPoint(i).mapPoint;
			double d1 = init.distance(pixel.getX(), pixel.getY());
			
			if(d1 < distance){
				distance = d1;
				posPoint = i;
				isPixel = true;
			}
			d1 = init.distance(map.getX(), map.getY());
			if(d1 < distance){
				distance = d1;
				posPoint = i;
				isPixel = false;
			}
		}
			
	}
	
	/**
	 * Reimplementacin del mtodo mouseReleased de Behavior.
	 *
	 * @param e MouseEvent
	 *
	 * @throws BehaviorException Excepcin lanzada cuando el Maptool.
	 */
	public void release(PointEvent e) throws BehaviorException {
		
		if(posPoint != -1){
			//Seleccionamos en el dialogo el punto que se est moviendo
			((GeoreferencingDialog)lyrGeoRaster.getGeoDialog()).getSelectPointsPanel().changePoint(true, posPoint);
			
			if(isPixel){
				//Comprobamos que el punto siga dentro de la imagen
				Point2D pixelPoint = lyrGeoRaster.world2Img(viewPort.toMapPoint(e.getPoint()));
				if(pixelPoint == null){
					JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(),
							PluginServices.getText(this, "fuera_de_extent"));
					return;
				}
				
				if(theView != null && lyrGeoRaster != null && lyrPoints != null)
					lyrGeoRaster.getFLyrPoints().getPointManager().
							updateData(posPoint + 1,
										pixelPoint, 
										null, 
										lyrGeoRaster.getGeoDialog(),
										theView);
				
			}else{
			 	Point2D mapPoint = viewPort.toMapPoint(e.getPoint());
				if(theView != null && lyrGeoRaster != null && lyrPoints != null)
					lyrGeoRaster.getFLyrPoints().getPointManager().
							updateData(posPoint + 1,
										null, 
										mapPoint, 
										lyrGeoRaster.getGeoDialog(),
										theView);
			}
			
		}
	}
}
