/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.gui.panels;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;

import org.gvsig.georeferencing.gui.dialog.GeoreferencingDialog;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.ViewPort;

public class AdjustGeorefPanel extends JPanel{

	//**********************Vars**********************************
	private int 					widthAdjustPanel = 380;
	private int 					heightAdjustPanel = 172;
	private JPanel 					pDetailImage = null;
	private ZoomControlPanel 		pixelZoom = null;
	private ZoomControlPanel 		mapZoom = null;
	private JPanel 					zoomx = null;
	private GeoreferencingDialog 	grd = null;
	private OptionsPanel 			optionsPanel = null;
	private JPanel 					pZoomxRight = null;
	private JPanel 					pZoomxLeft = null;
	private JLabel 					lzoom1 = null;
	private JLabel 					lzoom2 = null;
	private JLabel 					lzoomx1 = null;
	private JLabel 					lzoomx2 = null;
	//**********************Methods**************************************
	/**
	 * This is the default constructor
	 */
	public AdjustGeorefPanel(GeoreferencingDialog grd) {
		super();
		this.grd = grd;
		initialize();
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
        GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
        gridBagConstraints11.gridx = 0;
        gridBagConstraints11.gridy = 1;
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        this.setLayout(new GridBagLayout());
        this.setPreferredSize(new java.awt.Dimension(widthAdjustPanel, heightAdjustPanel));
        this.setSize(new java.awt.Dimension(widthAdjustPanel, heightAdjustPanel));
        this.setLocation(new java.awt.Point(0,0));
        
		this.add(getPDetailImage(), gridBagConstraints);		
		this.add(getPZoomx(), gridBagConstraints11);
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPDetailImage() {
		if (pDetailImage == null) {
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new java.awt.Insets(10,0,0,0);
			gridBagConstraints1.gridy = 0;
			gridBagConstraints1.gridx = 0;
			GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
			gridBagConstraints2.insets = new java.awt.Insets(10,0,0,0);
			gridBagConstraints2.gridy = 0;
			gridBagConstraints2.gridx = 1;
			pDetailImage = new JPanel();
			pDetailImage.setLayout(new GridBagLayout());
			pDetailImage.setPreferredSize(new java.awt.Dimension(widthAdjustPanel, 152));
			pDetailImage.add(getZoomLeft(), gridBagConstraints1);
			pDetailImage.add(getZoomRight(), gridBagConstraints2);	
		}
		return pDetailImage;
	}

	/**
	 * This method initializes ZoomControlPanel	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public ZoomControlPanel getZoomLeft() {
		if (pixelZoom == null) {
			pixelZoom = new ZoomControlPanel(true, grd);
		}
		return pixelZoom;
	}
	
	/**
	 * This method initializes jButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	public ZoomControlPanel getZoomRight() {
		if (mapZoom == null) {
			mapZoom = new ZoomControlPanel(false, grd);
		}
		return mapZoom;
	}

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	public JPanel getPZoomx() {
		if (zoomx == null) {
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.insets = new java.awt.Insets(0,0,0,0);
			gridBagConstraints4.gridy = 0;
			gridBagConstraints4.gridx = 1;
			GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
			gridBagConstraints3.gridx = 0;
			gridBagConstraints3.ipadx = 0;
			gridBagConstraints3.gridy = 0;
			zoomx = new JPanel();
			zoomx.setLayout(new GridBagLayout());
			zoomx.setPreferredSize(new java.awt.Dimension(widthAdjustPanel, 20));
			zoomx.add(getPZoomxRight(), gridBagConstraints3);
			zoomx.add(getPZoomxLeft(), gridBagConstraints4);
		}
		return zoomx;
	}
	
	/**
	 * Asigna los viewPort a los canvas para el dibujado de la minimagen 
	 * @param vp ViewPort asignado.
	 */
	public void setViewPort(ViewPort vp, double zoom){
		this.getZoomLeft().setViewPort(vp, zoom);
		this.getZoomRight().setViewPort(vp, zoom);
	}
	
	/**
	 * Realiza el dibujado de la minimagen de los cavas con el viewport actual 
	 */
	public void draw(){
		this.getZoomLeft().draw();
		this.getZoomRight().draw();
	}

	/**
	 * @return Returns the optionsPanel.
	 */
	public OptionsPanel getOptionsPanel() {
		return optionsPanel;
	}

    /**
     * Activa o desactiva este panel y todos los que lo componen
     * @param enabled variable booleana para la activacin y/o desactivacin
     */
    public void setEnabled(boolean enabled){
    	if(pixelZoom != null)
    		pixelZoom.setEnabled(enabled);
    	if(mapZoom != null)
    		mapZoom.setEnabled(enabled);
    }
	//**********************End Methods**********************************	

	/**
	 * This method initializes jPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPZoomxRight() {
		if (pZoomxRight == null) {
			FlowLayout flowLayout = new FlowLayout();
			flowLayout.setHgap(2);
			flowLayout.setAlignment(java.awt.FlowLayout.LEFT);
			flowLayout.setVgap(2);
			lzoomx1 = new JLabel();
			lzoomx1.setText("1:");
			lzoom1 = new JLabel();
			lzoom1.setText(PluginServices.getText(this, "escala") + ": ");
			pZoomxRight = new JPanel();
			pZoomxRight.setLayout(flowLayout);
			pZoomxRight.setPreferredSize(new java.awt.Dimension(210,20));
			pZoomxRight.add(lzoom1, null);
			pZoomxRight.add(lzoomx1, null);
		}
		return pZoomxRight;
	}

	/**
	 * This method initializes jPanel1	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getPZoomxLeft() {
		if (pZoomxLeft == null) {
			FlowLayout flowLayout1 = new FlowLayout();
			flowLayout1.setAlignment(java.awt.FlowLayout.LEFT);
			flowLayout1.setVgap(2);
			flowLayout1.setHgap(2);
			lzoomx2 = new JLabel();
			lzoomx2.setText("1:");
			lzoom2 = new JLabel();
			lzoom2.setText(PluginServices.getText(this, "escala") + ": ");
			pZoomxLeft = new JPanel();
			pZoomxLeft.setLayout(flowLayout1);
			pZoomxLeft.setPreferredSize(new java.awt.Dimension(170,20));
			pZoomxLeft.add(lzoom2, null);
			pZoomxLeft.add(lzoomx2, null);
		}
		return pZoomxLeft;
	}
	
	/**
	 * Asigna el zoom de la miniimagen con los controles a la derecha
	 * @param txt
	 */
	public void setLZoomRight(String txt){
		if(lzoomx1 != null)
			lzoomx1.setText(txt);
	}
	
	/**
	 * Asigna el zoom de la miniimagen con los controles a la izquierda
	 * @param txt
	 */
	public void setLZoomLeft(String txt){
		if(lzoomx2 != null)
			lzoomx2.setText(txt);
	}

}
