/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.gvsig.georeferencing.utils;

import java.util.ArrayList;

import org.cresques.px.Extent;

import com.iver.cit.gvsig.fmap.layers.IStackZoom;

/**
 * Clase para almacenar los extents que se van seleccionando al manipular
 * una imagen para su georreferenciacin. Esto permite recuperar extents anteriores.
 * 
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class StackZoom implements IStackZoom{
		
	//**********************Vars****************************************
	/**
	 * Nmero mximo de elementos en la pila
	 */
	private static final int		MAX = 10;
	private ArrayList 				extentList = new ArrayList();
	private Extent 					initExtent = null;	
	private int 					selectElem = -1;
	private int						top = -1;
	//**********************End Vars************************************
	
	//**********************Methods*************************************
	public StackZoom(){}
	
	/**
	 * Vacia la pila de Extents
	 */
	public void clear(){
	  extentList.clear();	
	  selectElem = 0;
	}
	//**********************End Methods**********************************
	
	//**********************Setters & Getters****************************
	/**
	 * Aade un extent a la lista
	 * @param ext
	 */
	public void setZoom(Extent ext){
		if(selectElem < (MAX - 1)){
			if(selectElem == top)
				top++;
			else
				top = selectElem + 1;
			selectElem ++;
			extentList.add(selectElem, ext);
		}
	}
		
	/**
	 * Recupera el ltimo extent aplicado. 
	 * <P>Si la pila est vacia o est situada en el primer
	 * elemento devuelve null sino devuelve el Extent guardado anterior al actual.</P>
	 * @return Extent anterior.
	 */
	public Extent getLastZoom(){
		if(extentList.size() == 0)
			return null;
		
		if(selectElem  == -1)
			return null;
		
		selectElem --;
		
		Extent ex = null;
		try{	
			ex = (Extent)extentList.get(selectElem);
		}catch (ArrayIndexOutOfBoundsException exc){
			//Elemento recuperado valdr null y colocamos el puntero en el primer elemento
			selectElem = 0;
			System.err.println("Zoom Stack Empty");
		}
		
		return ex;
	}
	
	/**
	 * Recupera el siguiente zoom. Se usa cuando se ha recuperado el ltimo
	 * zoom y se quiere volver al siguiente.
	 * <P>Si estamos en el tope de la pila o hemos llegado a su mxima capacidad o no tiene
	 * elementos devuelve null sino devuelve el siguiente</P>
	 *   
	 * @return
	 */
	public Extent getNextZoom(){
		if(	selectElem == (MAX -1) ||
			selectElem == top || 
			extentList.size() == 0 )
			return null;
		
		selectElem ++;
		
		Extent ex = null;
		try{
			ex = (Extent)extentList.get(selectElem);
		}catch (ArrayIndexOutOfBoundsException exc){
			//No hacemos nada y el elemento recuperado valdr null
			System.err.println("Zoom Stack On Top");
		}
		return ex;
	}
	
	/**
	 * @return Returns the initExtent.
	 */
	public Extent getInitExtent() {
		return initExtent;
	}
	
	/**
	 * @param initExtent The initExtent to set.
	 */
	public void setInitExtent(Extent initExtent, boolean incr) {
		if(incr)
			selectElem ++;
		extentList.add(0, initExtent);
		this.initExtent = initExtent;
	}
		
	/**
	 * Obtiene el apuntador a elementos
	 * @return 
	 */
	public int getSelectElem() {
		return selectElem;
	}
	
	/**
	 * Obtiene el nmero de elementos de la pila
	 * @return Nmero de elementos de la pila
	 */
	public int getStackSize(){
		return top;
	}
	//**********************End Setters & Getters************************
}
