package com.iver.cit.gvsig.gui.wizards;

import java.awt.Component;
import java.io.IOException;
import java.net.ConnectException;
import java.net.URL;

import javax.swing.JOptionPane;

import com.iver.andami.PluginServices;
import com.iver.cit.gvsig.fmap.DriverException;
import com.iver.cit.gvsig.fmap.drivers.WFSException;
import com.iver.cit.gvsig.fmap.drivers.wfs.FMapWFSDriver;
import com.iver.cit.gvsig.fmap.drivers.wfs.FMapWFSDriverFactory;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: WFSWizardData.java,v 1.7.2.2 2006/11/17 11:28:45 ppiqueras Exp $
 * $Log: WFSWizardData.java,v $
 * Revision 1.7.2.2  2006/11/17 11:28:45  ppiqueras
 * Corregidos bugs y añadida nueva funcionalidad.
 *
 * Revision 1.9  2006/11/16 16:57:30  jorpiell
 * Un mensaje de error que no se mostraba
 *
 * Revision 1.8  2006/11/16 13:28:50  jorpiell
 * Se usa la factoria de layers para crear la capa
 *
 * Revision 1.7  2006/07/11 07:26:43  caballero
 * traduccin cant_connect_wfs
 *
 * Revision 1.6  2006/06/14 08:46:24  jorpiell
 * Se tiene en cuanta la opcion para refrescar las capabilities
 *
 * Revision 1.5  2006/05/23 13:21:59  jorpiell
 * Si hay algn problema en la carga se muestra un mensaje de error
 *
 * Revision 1.4  2006/05/23 08:09:53  jorpiell
 * Se ha cambiado la forma en la que se leian los valores seleccionados en los paneles y se ha cambiado el comportamiento de los botones
 *
 * Revision 1.3  2006/05/19 12:58:03  jorpiell
 * Modificados algunos paneles
 *
 * Revision 1.2  2006/04/20 16:38:24  jorpiell
 * Ahora mismo ya se puede hacer un getCapabilities y un getDescribeType de la capa seleccionada para ver los atributos a dibujar. Queda implementar el panel de opciones y hacer el getFeature().
 *
 * Revision 1.1  2006/04/19 12:50:16  jorpiell
 * Primer commit de la aplicacin. Se puede hacer un getCapabilities y ver el mensaje de vienvenida del servidor
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class WFSWizardData {
	private FMapWFSDriver driver = null;
	private String title = null;
	private String _abstract = null;
	private String serverVersion = null;
	private int buffer = 0;
	private int timeOut = 0;
	private String UserName = null;

	public String getOnlineResource(){
		return driver.getOnlineResource();
	}

	/**
	 *
	 * @return The host name
	 */
	public String getHost(){
		return driver.getHost();
	}

	/**
	 * Create the WFSClient and try to connect
	 * @param host
	 * Server name
	 * @throws DriverException
	 */
	public void setHost(URL host,boolean override) throws DriverException{
		try {
			driver = FMapWFSDriverFactory.getFMapDriverForURL(host);
			driver.createClient(host);

			if (!driver.connect(override,null)){
				throw new DriverException(PluginServices.getText(this, "cant_connect_wfs") + host.toString());
			}

		} catch (ConnectException e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(), PluginServices.getText(this,"server_timeout"));
			e.printStackTrace();
			return;
		} catch (IOException e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(), PluginServices.getText(this, "cant_connect"));
			e.printStackTrace();
			return;
		} catch (WFSException e) {
			JOptionPane.showMessageDialog((Component)PluginServices.getMainFrame(), PluginServices.getText(this, "cant_connect"));
			e.printStackTrace();
			return;
		}
	}

	/**
	 * Server's title (not used in gvSIG)
	 *
	 * Ttulo del servidor (no se usa en gvSIG)
	 *
	 * @return
	 */
	public String getTitle() {
		title = driver.getServiceTitle();
		if (title == null){
			return "None";
		}
		return title;
	}

	/**
	 * The server description.
	 *
	 * La descripcin del servidor
	 *
	 * @return String
	 */
	public String getAbstract() {
		_abstract= driver.getServiceAbstract();
		if (_abstract == null){
			return "None";
		}
		return _abstract;
	}

	/**
	 * @return Returns the driver.
	 */
	public FMapWFSDriver getDriver() {
		return driver;
	}

	/**
	 *
	 * @return Features vector
	 */
	public Object[] getFeatures(){
		return driver.getLayerList();
	}

	/**
	 * Return the feature info
	 * @param featureName
	 * @return
	 */
	public Object getFeatureInfo(String featureName){
		return driver.getLayerInfo(featureName);
	}

	/**
	 *
	 * @return server type
	 */
	public String getServerType() {
		serverVersion = driver.getVersion();
		if (serverVersion==null){
			return "WFS";
		}
		return "WFS "+ serverVersion;
	}

	/**
	 * @return Returns the buffer.
	 */
	public int getBuffer() {
		return buffer;
	}

	/**
	 * @param buffer The buffer to set.
	 */
	public void setBuffer(int buffer) {
		this.buffer = buffer;
	}

	/**
	 * @return Returns the timeOut.
	 */
	public int getTimeOut() {
		return timeOut;
	}

	/**
	 * @param timeOut The timeOut to set.
	 */
	public void setTimeOut(int timeOut) {
		this.timeOut = timeOut;
	}

	/**
	 * @return Returns the userName.
	 */
	public String getUserName() {
		return UserName;
	}

	/**
	 * @param userName The userName to set.
	 */
	public void setUserName(String userName) {
		UserName = userName;
	}



}
