/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.filter.statistics;

import java.awt.Image;
import java.awt.image.BufferedImage;

import org.cresques.filter.RasterBuf;


/**
 * Clase base para los filtros de calculo de mximo y mnimo en sus diferentes
 * tipos de datos.
 */
public class ComputeMinMaxImageFilter extends ComputeMinMaxFilter {
    /**
     * Constructor
     *
     */
    public ComputeMinMaxImageFilter() {
        super();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
        //Obtencin de parmetros
        this.image = (Image) params.get("raster");

        height = image.getHeight(null);
        width = image.getWidth(null);
        super.pre();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#process(int, int)
     */
    public void process(int x, int y) {
        int px = ((BufferedImage) image).getRGB(x, y);

        int pxBand = ((px & 0xff0000) >> 16);

        if (pxBand < min[0]) {
            secondMin[0] = min[0];
            min[0] = pxBand;
        } else if ((pxBand < secondMin[0]) && (pxBand != min[0])) {
            secondMin[0] = pxBand;
        }

        if (pxBand > max[0]) {
            secondMax[0] = max[0];
            max[0] = pxBand;
        } else if ((pxBand > secondMax[0]) && (pxBand != max[0])) {
            secondMax[0] = pxBand;
        }

        pxBand = ((px & 0x00ff00) >> 8);

        if (pxBand < min[1]) {
            secondMin[1] = min[1];
            min[1] = pxBand;
        } else if ((pxBand < secondMin[1]) && (pxBand != min[1])) {
            secondMin[1] = pxBand;
        }

        if (pxBand > max[1]) {
            secondMax[1] = max[1];
            max[1] = pxBand;
        } else if ((pxBand > secondMax[1]) && (pxBand != max[1])) {
            secondMax[1] = pxBand;
        }

        pxBand = (px & 0x0000ff);

        if (pxBand < min[2]) {
            secondMin[2] = min[2];
            min[2] = pxBand;
        } else if ((pxBand < secondMin[2]) && (pxBand != min[2])) {
            secondMin[2] = pxBand;
        }

        if (pxBand > max[2]) {
            secondMax[2] = max[2];
            max[2] = pxBand;
        } else if ((pxBand > secondMax[2]) && (pxBand != max[2])) {
            secondMax[2] = pxBand;
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
        super.post();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }
}
