/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io;

import org.cresques.cts.IProjection;

import org.cresques.ui.CQApp;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.Iterator;
import java.util.Map;
import java.util.TreeMap;
import java.util.Vector;

import javax.swing.ImageIcon;
import javax.swing.JMenu;
import javax.swing.JMenuItem;


/**
 * @author "Luis W. Sevilla" <sevilla_lui@gva.es>
 */
class Bookmark {
    String folder;
    String name;
    String[] fName;
    String projName;
    IProjection proj;

    public Bookmark(String fld, String n, String f, IProjection p) {
        projName = null;
        fName = new String[1];
        folder = fld;
        name = n;
        fName[0] = f;
        proj = p;
    }

    public Bookmark(String fld, String n, String[] f, IProjection p) {
        projName = null;
        fName = new String[f.length];
        folder = fld;
        name = n;
        proj = p;

        for (int i = 0; i < f.length; i++)
            fName[i] = f[i];
    }
}


public class BookmarkList extends TreeMap implements ActionListener {
    final private static long serialVersionUID = -3370601314380922368L;
    CQApp app = null;

    public BookmarkList(CQApp mainApp) {
        app = mainApp;
        loadList();
    }

    public void loadList() {
    }

    public void addEntry(String fld, String n, String f, IProjection p) {
        addEntry(new Bookmark(fld, n, f, p));
    }

    public void addEntry(String fld, String n, String[] f, IProjection p) {
        addEntry(new Bookmark(fld, n, f, p));
    }

    public void addEntry(Bookmark bookmark) {
        Vector vector;

        if (containsKey(bookmark.folder)) {
            vector = (Vector) get(bookmark.folder);
        } else {
            vector = new Vector();
        }

        vector.add(bookmark);
        put(bookmark.folder, vector);
    }

    public JMenu getJMenu() {
        JMenuItem it = null;
        JMenu menu = null;
        JMenu menu2 = null;
        ClassLoader loader = this.getClass().getClassLoader();

        //loader.getResource("images/"+"folderXP.gif");
        //System.err.println("URL="+loader.getResource("images/"+"folderXP.gif"));
        ImageIcon folderIcon = new ImageIcon(loader.getResource("images/" +
                                                                "folderXP.gif"));
        ImageIcon dxfIcon = new ImageIcon(loader.getResource("images/" +
                                                             "fileXPdxf.gif"));
        ImageIcon gmlIcon = new ImageIcon(loader.getResource("images/" +
                                                             "fileXPgml.gif"));
        ImageIcon ecwIcon = new ImageIcon(loader.getResource("images/" +
                                                             "fileXPecw.gif"));
        ImageIcon tifIcon = new ImageIcon(loader.getResource("images/" +
                                                             "fileXPtif.gif"));
        ImageIcon fileIcon = new ImageIcon(loader.getResource("images/" +
                                                              "fileXP.gif"));

        menu = new JMenu("Favoritos");
        menu.setMnemonic('F');

        // Arbol de carpetas
        TreeMap menuMap = new TreeMap();
        String[] menuNames;
        JMenu parentMenu = null;
        String mName = null;
        String key = null;
        ;

        Iterator mIter = keySet().iterator();

        while (mIter.hasNext()) {
            key = (String) mIter.next();
            System.out.println("Key:" + key);
            menuNames = key.split("\\|");

            parentMenu = menu;
            mName = "";

            for (int i = 0; i < menuNames.length; i++) {
                if (i > 0) {
                    mName += "|";
                }

                mName += menuNames[i];

                if (menuMap.containsKey(mName)) {
                    //System.out.println(" mName:"+mName+", parent:"+parentMenu.getText());
                    parentMenu = (JMenu) menuMap.get(mName);
                } else {
                    //System.out.println("+mName:"+mName+", parent:"+parentMenu.getText());
                    menu2 = new JMenu(menuNames[i]);
                    menu2.setIcon(folderIcon);
                    parentMenu.add(menu2);
                    menuMap.put(mName, menu2);
                    parentMenu = menu2;
                }
            }
        }

        // Entradas (Bookmarks)
        mIter = entrySet().iterator();

        while (mIter.hasNext()) {
            Map.Entry entry = (Map.Entry) mIter.next();
            String folder = (String) entry.getKey();

            /*menu2 = new JMenu(folder);
            menu2.setIcon(folderIcon);
            menu.add(menu2);*/
            menu2 = (JMenu) menuMap.get(folder);

            Vector vector = ((Vector) entry.getValue());
            Bookmark bookmark;

            for (int i = 0; i < vector.size(); i++) {
                bookmark = (Bookmark) vector.get(i);
                it = new JMenuItem(bookmark.name);
                it.setActionCommand(i + "|" + folder);
                it.addActionListener(this);

                if (bookmark.fName[0].endsWith("tif")) {
                    it.setIcon(tifIcon);
                } else if (bookmark.fName[0].endsWith("ecw")) {
                    it.setIcon(ecwIcon);
                } else if (bookmark.fName[0].endsWith("gml")) {
                    it.setIcon(gmlIcon);
                } else if (bookmark.fName[0].endsWith("dxf") ||
                               bookmark.fName[0].endsWith("DXF")) {
                    it.setIcon(dxfIcon);
                } else {
                    it.setIcon(fileIcon);
                }

                menu2.add(it);
            }
        }

        //menu.add(menu2);
        return menu;
    }

    public void actionPerformed(ActionEvent event) {
        IProjection proj = null;
        String cmd = event.getActionCommand();
        int mark = cmd.indexOf('|');
        int vectorPos = Integer.parseInt(cmd.substring(0, mark));
        cmd = cmd.substring(mark + 1);

        Bookmark bookmark = (Bookmark) ((Vector) get(cmd)).get(vectorPos);

        if (bookmark.fName.length == 1) {
            app.loadFile(bookmark.fName[0], bookmark.proj);
        } else {
            app.loadFile(bookmark.fName, bookmark.proj);
        }
    }
}
