/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;

import org.cresques.px.IObjList;
import org.cresques.px.PxObjList;
import org.cresques.px.gml.LineString;

import java.io.DataInputStream;
import java.io.EOFException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;


/**
 * @author luisw
 */
public class TrkFile extends GeoFile {
    private DataInputStream is;
    long l = 0;
    LineString line;

    public TrkFile(IProjection proj, String name) {
        super(proj, name);
    }

    /* (no Javadoc)
     * @see org.cresques.io.GeoFile#load()
     */
    public GeoFile load() {
        try {
            return load(new FileInputStream(name));
        } catch (NumberFormatException e) {
            // TODO Bloque catch generado automticamente
            e.printStackTrace();
        } catch (FileNotFoundException e) {
            // TODO Bloque catch generado automticamente
            e.printStackTrace();
        } catch (Exception e) {
            // TODO Bloque catch generado automticamente
            e.printStackTrace();
        }

        return this;
    }

    public GeoFile load(InputStream is) throws NumberFormatException, Exception {
        System.out.println("Trk: Cargando '" + name + "' ...");
        this.is = new DataInputStream(is);
        line = new LineString();

        while (readPoint()) {
            l++;
        }

        extent.add(line.getExtent());
        is.close();
        System.out.println("Trk: '" + name + "' cargado. (" + l + " puntos).");

        return this;
    }

    public boolean readPoint() {
        int h;
        int m;
        int s;
        int sgn = 1;
        double D;
        double M;
        double S;
        double lat;
        double lng;

        try {
            h = is.readUnsignedByte();
            m = is.readUnsignedByte();
            s = is.readUnsignedByte();
            is.skipBytes(1);
            D = is.readUnsignedByte();
            M = is.readByte();
            S = is.readUnsignedShort();
            S /= 1000.0;

            if (M < 0) {
                sgn = -1;
                M *= -1;
            } else {
                sgn = 1;
            }

            lat = sgn * (D + ((M / 60D) + (S / 3600D)));
            D = is.readUnsignedByte();
            M = is.readByte();
            S = is.readUnsignedShort();
            S /= 1000.0;

            if (M < 0) {
                sgn = -1;
                M *= -1;
            } else {
                sgn = 1;
            }

            lng = sgn * (D + ((M / 60D) + (S / 3600D)));
            line.add(proj.createPoint(lng, lat));
        } catch (EOFException eof) {
            return false;
        } catch (IOException e) {
            // TODO Bloque catch generado automticamente
            e.printStackTrace();

            return false;
        }

        return true;
    }

    /* (no Javadoc)
     * @see org.cresques.io.GeoFile#reProject(org.cresques.geo.ReProjection)
     */
    public IObjList getObjects() {
        IObjList oList = new PxObjList(proj);
        oList.add(line);

        return oList;
    }

    public void reProject(ICoordTrans rp) {
        line.reProject(rp);
        setProjection(rp.getPDest());
    }

    /* (non-Javadoc)
     * @see org.cresques.io.GeoFile#close()
     */
    public void close() {
        // TODO Auto-generated method stub
    }
}
