package com.iver.cit.gvsig.fmap.drivers.gml;

import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.Date;

import org.apache.log4j.Logger;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;

import com.hardcode.gdbms.engine.values.ComplexValue;
import com.hardcode.gdbms.engine.values.Value;
import com.hardcode.gdbms.engine.values.ValueFactory;
import com.iver.cit.gvsig.fmap.core.DefaultFeature;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ShapeFactory;
import com.iver.cit.gvsig.fmap.core.v02.FConverter;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.GeometryFactory;
import com.vividsolutions.jts.io.gml2.GMLReader;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: FMAPGeometryFactory.java,v 1.1.4.1 2006/11/15 00:08:23 jjdelcerro Exp $
 * $Log: FMAPGeometryFactory.java,v $
 * Revision 1.1.4.1  2006/11/15 00:08:23  jjdelcerro
 * *** empty log message ***
 *
 * Revision 1.2  2006/10/10 12:55:27  jorpiell
 * Se ha aadido el soporte de features complejas
 *
 * Revision 1.1  2006/08/10 12:03:43  jorpiell
 * Se usa el nuevo driver de GML de remoteServices
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class FMAPGeometryFactory implements IGeometriesFactory{
	private static Logger logger = Logger.getLogger(FMAPGeometryFactory.class);
	
	public Object createPoint2D(double x, double y){
		return ShapeFactory.createPoint2D(x,y);
	}

	public Object createPoint2D(Point2D point2D) {
		return createPoint2D(point2D.getX(),point2D.getY());
	}
	
	public Object createMultipoint2D(double[] x, double[] y){
		return ShapeFactory.createMultipoint2D(x,y);
	}
	
	public Object createSimpleFeature(String element, Object geometry, ArrayList params, ArrayList values) {
		ArrayList fmapParams = getParams((ArrayList)params.get(1));
		Value[] fmapValues = getValues((ArrayList)params.get(1),(ArrayList)values.get(1));
		IFeature feature = new DefaultFeature((IGeometry)geometry,fmapValues);
		return new FeatureWithAttributes(feature,fmapParams,fmapValues);
	}

	public Object createGeometry(String gmlGeometry) {
		GMLReader reader = new GMLReader();
		try {
			Geometry geom = reader.read(gmlGeometry,new GeometryFactory());
			return ShapeFactory.createGeometry(FConverter.jts_to_java2d(geom));
		} catch (Exception e){
			logger.error("Can't parse: " + gmlGeometry,e);
		}
		return null;
	}
	
	private ArrayList getParams(ArrayList params){
		ArrayList fmapParams = new ArrayList();
		for (int i=0 ; i<params.size() ; i++){
			if (!(params.get(i) instanceof ArrayList)){
				 fmapParams.add(params.get(i));
			}
		}
		return fmapParams;
	}
	
	private Value[] getValues(ArrayList params,ArrayList values){
		int i = 0;
		ArrayList fmapValues = new ArrayList();
		while(i<params.size()){
			if ((i+1 < params.size()) && (params.get(i+1) instanceof ArrayList)){
				ComplexValue value = ValueFactory.createComplexValue((String)params.get(i));
				ArrayList subParams = getParams((ArrayList)params.get(i+1));
				Value[] subValues = getValues((ArrayList)params.get(i+1),(ArrayList)values.get(i+1));
				for (int j=0 ; j<subParams.size() ; j++){
					value.put(subParams.get(j),subValues[j]);
				}
				fmapValues.add(value);
				i = i + 2;
			}else{
				fmapValues.add(getValue(values.get(i)));
				i++;
			}				
		}
		Value[] returnValues = new Value[fmapValues.size()];
		for (int k=0 ; k<fmapValues.size() ; k++){
			returnValues[k] = (Value)fmapValues.get(k);
		}
		return returnValues;
	}
	
	/**
	 * Gets the attributes
	 * @param attr
	 * @return
	 */
	private Value getValue(Object attr) {	
		if (attr instanceof Double){
			return ValueFactory.createValue(((Double)attr).doubleValue());
		}else if (attr instanceof String){
			return ValueFactory.createValue(String.valueOf(attr));
		}else if (attr instanceof Long){
			return ValueFactory.createValue(((Long)attr).longValue());
		}else if (attr instanceof Integer){
			return ValueFactory.createValue(((Integer)attr).intValue());
		}else if (attr instanceof Float){
			return ValueFactory.createValue(((Float)attr).floatValue());
		}else if (attr instanceof Short){
			return ValueFactory.createValue(((Short)attr).shortValue());
		}else if (attr instanceof Boolean){
			return ValueFactory.createValue(((Boolean)attr).booleanValue());
		}else if (attr instanceof Date){
			return ValueFactory.createValue(((Date)attr));
		}
		return ValueFactory.createValue(((String)""));
	}
	
	public class FeatureWithAttributes{
		private IFeature feature;
		private ArrayList attributeName;
		private Value[] attributeValue;
		
		public FeatureWithAttributes(IFeature feature, ArrayList attributeName, Value[] attributeValue) {
			super();		
			this.feature = feature;
			this.attributeName = attributeName;
			this.attributeValue = attributeValue;
		}

		/**
		 * @return Returns the attributeName.
		 */
		public ArrayList getAttributeName() {
			return attributeName;
		}

		/**
		 * @return Returns the feature.
		 */
		public IFeature getFeature() {
			return feature;
		}

		/**
		 * @return Returns the attributeValue.
		 */
		public Value[] getAttributeValue() {
			return attributeValue;
		}
		
	}

}
