package com.iver.cit.gvsig.fmap.edition.writers.gml;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import org.cresques.cts.IProjection;
import org.geotools.xml.gml.GMLSchema;

import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.drivers.FieldDescription;
import com.iver.cit.gvsig.fmap.edition.EditionException;
import com.iver.utiles.StringUtilities;
import com.vividsolutions.jts.geom.Geometry;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLFileCreator.java,v 1.2.4.1 2006/11/20 08:11:44 jorpiell Exp $
 * $Log: GMLFileCreator.java,v $
 * Revision 1.2.4.1  2006/11/20 08:11:44  jorpiell
 * Se ha sustituido el carcter & por &amp;. El parser de XML lo requiere
 *
 * Revision 1.2  2006/08/10 12:03:43  jorpiell
 * Se usa el nuevo driver de GML de remoteServices
 *
 * Revision 1.1  2006/07/19 12:29:39  jorpiell
 * Aadido el driver de GML
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLFileCreator {
	private String schemaFileName = null;
	private Writer writer = null;
	private com.vividsolutions.jts.io.gml2.GMLWriter featureWriter = null;
	private IProjection projection = null;
	private String srsName = null;
	private Rectangle2D extent = null;
	private File m_File = null;
	private FieldDescription[] attributes = null;
		
	public GMLFileCreator(File file,String schemaFileName) throws IOException{
		this.m_File = file;
		this.schemaFileName = schemaFileName;
	}
	
	/**
	 * This method creates the GML head file. It must be 
	 * called before start to add geometries.
	 * @throws IOException
	 */
	public void initialize() throws IOException{		
		writer = new FileWriter(m_File);
		writer.write(getInitFile());
		writer.write(getBoundedBy());
		featureWriter = new com.vividsolutions.jts.io.gml2.GMLWriter();
		featureWriter.setPrefix("gml");
	}
	
	/**
	 * Intert a new Feature
	 * @param geom
	 * @throws IOException
	 */
	public void insertFeature(Geometry geom,Value[] attributes) throws IOException{
			writer.write(getInitfeature());
			writer.write(getInitGeomAttribute());
			featureWriter.write(geom,writer);
			writer.write(getEndGeomAttribute());
			
			for (int i=0 ; i<attributes.length ; i++){
				String attributeName = this.attributes[i].getFieldName();
				String attibuteValue = StringUtilities.replace(attributes[i].toString(),"&","&amp;");
				writer.write(getAttribute(attributeName,attibuteValue));
			}
			
			writer.write(getEndFeature());	
	}
	
	

	/**
	 * Writes the GML file
	 * @throws IOException 
	 *
	 */
	public void writeFile() throws IOException {
		writer.write(getEndFile());
		writer.close();		
	}

	/**
	 * Sets the projection
	 * @param extent
	 * @param proj
	 */
	public void setBoundedBy(Rectangle2D extent, IProjection proj) {
		this.extent = extent;
		this.projection = proj;		
	}
	
	/**
	 * Sets the attributes collection
	 * @param attributes
	 */
	public void setAttibutes(FieldDescription[] attributes){
		this.attributes = attributes;
	}
	
	/**
	 *  Creates an init Tag for a feature member
	 * @return
	 */
	private String getInitfeature(){
		StringBuffer string = new StringBuffer();
		string.append("<gml:featureMember>\n");
		string.append("<" + GMLSchemaCreator.NAMESPACE + ":" + GMLSchemaCreator.getFeatureName() + ">");
		return string.toString();
		
	}
	
	/**
	 * Creates an end Tag for a feature member
	 * @return
	 */
	private String getEndFeature(){
		StringBuffer string = new StringBuffer();
		string.append("</" + GMLSchemaCreator.NAMESPACE + ":" + GMLSchemaCreator.getFeatureName() + ">");
		string.append("</gml:featureMember>");
		return string.toString();
	}
	
	/**
	 * Creates the GML File first line
	 * @param fileName
	 * GML file name used to create the schema name
	 * @return
	 */
	private String getInitFile(){
		StringBuffer string = new StringBuffer();
		string.append("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>");
		string.append("<gml:FeatureCollection ");
		string.append("xmlns:" + GMLSchemaCreator.NAMESPACE + "=\"" + GMLSchemaCreator.TARGET_NAMESPACE + "\" ");
		string.append("xmlns:gml=\"http://www.opengis.net/gml\" ");
		string.append("xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" "); 
		string.append("xsi:schemaLocation=\"" + GMLSchemaCreator.TARGET_NAMESPACE + " " + schemaFileName + "\"");
		string.append(">");
		return string.toString();
	}
	
	/**
	 * Creates the GML Attributes description
	 * @return
	 */
	private String getInitGeomAttribute(){
		StringBuffer string = new StringBuffer();
		string.append("<" + GMLSchemaCreator.NAMESPACE + ":" + GMLSchemaCreator.GML_GEOMETRY + ">");
		return string.toString();
	}
	
	/**
	 * Creates the GML attribute code
	 * @param attributeName
	 * @param attributeValue
	 * @return
	 */
	private String getAttribute(String attributeName, String attributeValue){
		StringBuffer string = new StringBuffer();
		string.append("<" + GMLSchemaCreator.NAMESPACE + ":" + attributeName + ">");
		string.append(attributeValue);
		string.append("</" + GMLSchemaCreator.NAMESPACE + ":" + attributeName + ">\n");
		return string.toString();
	}
	
	/**
	 * Creates the GML Attributes description
	 */
	private String getEndGeomAttribute(){
		StringBuffer string = new StringBuffer();
		string.append("</" + GMLSchemaCreator.NAMESPACE + ":" + GMLSchemaCreator.GML_GEOMETRY + ">\n");
		return string.toString();
	}

	/**
	 * Creates the GML File last line
	 * @return
	 */
	private String getEndFile(){
		StringBuffer string = new StringBuffer();
		string.append("</gml:FeatureCollection>");
		return string.toString();
	}
	
	/**
	 * Creates the boundedBy param
	 * @return
	 */
	private String getBoundedBy(){
		srsName = GMLBoundedBy.fromGvSigToGML(projection);
		StringBuffer string = new StringBuffer();
		string.append("<gml:boundedBy>");
	    string.append("<gml:Box srsName=\"" + srsName + "\">");
	    string.append("<gml:coordinates decimal=\".\" cs=\",\" ");
	    string.append("ts=\" \">" + extent.getMinX() + "," + extent.getMinY() + " ");
	    string.append(extent.getMaxX() + "," + extent.getMaxY() + "</gml:coordinates>");
	    string.append("</gml:Box>");
	    string.append("</gml:boundedBy>");
	    return string.toString();	    
	}
	
	

}
