/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

import com.iver.cit.gvsig.fmap.FMapEvent;


/**
 * Eventos de capa
 *
 * @author Fernando Gonzlez Corts
 */
public class LayerEvent extends FMapEvent {
	private String property;
	private FLayer source;
	private static final int VISIBILITY_CHANGED = 0;
	private static final int ACTIVATION_CHANGED = 1;
	private static final int NAME_CHANGED = 2;
	private static final int EDITION_CHANGED = 3;

	public static LayerEvent createVisibilityChangedEvent(FLyrDefault default1, String property){
		return new LayerEvent(default1, property, VISIBILITY_CHANGED);
	}
	
	public static LayerEvent createActivationChangedEvent(FLyrDefault default1, String property){
		return new LayerEvent(default1, property, ACTIVATION_CHANGED);
	}
	
	public static LayerEvent createNameChangedEvent(FLyrDefault default1, String property){
		return new LayerEvent(default1, property, NAME_CHANGED);
	}
	
	public static LayerEvent createEditionChangedEvent(FLyrDefault default1, String property){
		return new LayerEvent(default1, property, EDITION_CHANGED);
	}
	
	/**
	 * Crea un nuevo LayerEvent.
	 *
	 * @param default1
	 * @param property
	 */
	private LayerEvent(FLayer default1, String property, int eventType) {
		source = default1;
		this.property = property;
		setEventType(eventType);
	}

	/**
	 * Devuelve el FLayer.
	 *
	 * @return FLayer.
	 */
	public FLayer getSource() {
		return source;
	}

	/**
	 * Inserta el FLayer.
	 *
	 * @param source FLayer.
	 */
	public void setSource(FLayer source) {
		this.source = source;
	}

	/**
	 * Devulve las propiedades.
	 *
	 * @return String.
	 */
	public String getProperty() {
		return property;
	}

	/**
	 * Inserta las Propiedades.
	 *
	 * @param property String.
	 */
	public void setProperty(String property) {
		this.property = property;
	}
}
