/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers;

/**
 * Evento de posicin de la capa en el TOC.
 *
 * @author Vicente Caballero Navarro
 */
public class LayerPositionEvent extends LayerCollectionEvent {
	/**
	 * Indica el indice que tena la capa afectada en caso de que el evento sea
	 * un layerMoved. En caso de que no sea dicho evento, tendra el mismo
	 * valor que el atributo index
	 */
	private int oldPos;

	/** Indice de la capa en la coleccion */
	private int newPos;

	private final static int LAYER_MOVING = 4;

	private final static int LAYER_MOVED = 1;


	public static LayerPositionEvent createLayerMovedEvent(FLayer lyr, int oldp, int newp){
		return new LayerPositionEvent(lyr, oldp, newp, LAYER_MOVED);
	}

	public static LayerPositionEvent createLayerMovingEvent(FLayer lyr, int oldp, int newp){
		return new LayerPositionEvent(lyr, oldp, newp, LAYER_MOVING);
	}

	/**
	 * Crea un nuevo LayerPositionEvent.
	 *
	 * @param lyr
	 * @param oldp Antigua posicin.
	 * @param newp Nueva posicin.
	 */
	private LayerPositionEvent(FLayer lyr, int oldp, int newp, int eventType) {
		super(lyr, eventType);
		oldPos = oldp;
		newPos = newp;
	}

	/**
	 * Devuelve la posicin origen.
	 *
	 * @return Posicin origen.
	 */
	public int getOldPos() {
		return oldPos;
	}

	/**
	 * Devuelve la posicin destino.
	 *
	 * @return Posicin destino.
	 */
	public int getNewPos() {
		return newPos;
	}
}
