package com.hardcode.gdbms.engine.data;

import com.hardcode.gdbms.engine.data.driver.DriverException;

import java.util.HashMap;


/**
 * Implementacin del soporte para acceso a campos por nombre
 *
 * @author Fernando Gonzlez Corts
 */
public class FieldNameAccessSupport {
    private HashMap nameIndex = null;
    private DataSource dataSource;

    /**
     * Creates a new FieldNameAccessSupport object.
     *
     * @param ds DataSource con los campos sobre los que se accede
     */
    public FieldNameAccessSupport(DataSource ds) {
        dataSource = ds;
    }

    /**
     * @see com.hardcode.gdbms.engine.data.FieldNameAccess#getFieldIndexByName(java.lang.String)
     */
    public int getFieldIndexByName(String fieldName) throws DriverException {
        Integer i = (Integer) getNameIndex().get(fieldName.toUpperCase());

        if (i == null) {
            return -1;
        }

        return i.intValue();
    }

    /**
     * Obtiene una referencia a la tabla de indices por nombre, creando dicha
     * tabla si es la primera vez que se accede
     *
     * @return tabla indice-nombre
     *
     * @throws DriverException Si hay error accediendo al DataSource
     */
    private HashMap getNameIndex() throws DriverException {
        if (nameIndex == null) {
            //Este metodo se invocar con el DataSource abierto ya
            nameIndex = new HashMap();

            for (int i = 0; i < dataSource.getFieldCount(); i++) {
                nameIndex.put(dataSource.getFieldName(i).toUpperCase(), new Integer(i));
            }
        }

        return nameIndex;
    }
}
