package org.gvsig.remoteClient.gml;

import java.awt.geom.Rectangle2D;
import java.io.File;
import java.io.InputStream;
import java.util.Hashtable;

import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.gvsig.remoteClient.utils.CapabilitiesTags;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLFeaturesParser.java,v 1.1 2006/08/10 12:00:49 jorpiell Exp $
 * $Log: GMLFeaturesParser.java,v $
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * This class has all the methods that a GML parser
 * must implement. It has methods to parse some data and
 * methods to get info about the GML file
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public abstract class GMLFeaturesParser {
	private XMLSchemaParser parser = null;
	private IGeometriesFactory factory = null;
	private String documentRoot = null;	
	private Extent extent = null;
		
	public GMLFeaturesParser(IGeometriesFactory factory,XMLSchemaParser parser) throws GMLException {
		super();
		this.factory = factory;
		this.parser = parser;
		initialize();
	}
	
	/**
	 * Gets one iterator to retrieve all the 
	 * GML features
	 * @param file
	 * File to parse
	 * @return
	 */
	public abstract IGMLFeaturesIterator getFeaturesReader() throws GMLException;	

	/**
	 * It initializes the GML file. It parses all the GML code
	 * that is located before the GML features
	 * @throws GMLException
	 */
	private void initialize() throws GMLException{
		parser.setSchema("gml");
		int currentTag;		
				
		try {
			currentTag = parser.getEventType();		
			switch(currentTag){
			case KXmlParser.START_TAG:
				documentRoot = parser.getName();
				break;
			case KXmlParser.END_TAG:				
					
				break;
			case KXmlParser.TEXT:                   
				break;
			}
		} catch (XmlPullParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_PARSE);
		} 
		initializeHeader();
	}	
	
	/**
	 * It initializes some GML fields like the BBOX, the imports,
	 * etc. 
	 * @throws GMLException
	 */
	public abstract void initializeHeader() throws GMLException;


	/**
	 * @return Returns the extent.
	 */
	public Extent getExtent() {
		return extent;
	}

	/**
	 * @return Returns the documentRoot.
	 */
	public String getDocumentRoot() {
		return documentRoot;
	}

	/**
	 * @return Returns the factory.
	 */
	public IGeometriesFactory getFactory() {
		return factory;
	}

	/**
	 * @return Returns the parser.
	 */
	public XMLSchemaParser getParser() {
		return parser;
	}

	/**
	 * @param extent The extent to set.
	 */
	public void setExtent(Extent extent) {
		this.extent = extent;
	}	
	
	public static class Extent{
		private Rectangle2D extent = null;
		private String srs = null;
		
		public Extent(Rectangle2D extent, String srs) {
			super();			
			this.extent = extent;
			this.srs = srs;
		}
		/**
		 * @return Returns the extent.
		 */
		public Rectangle2D getExtent() {
			return extent;
		}
		/**
		 * @return Returns the srs.
		 */
		public String getSrs() {
			return srs;
		}
	}
	
}
