package org.gvsig.remoteClient.gml.factories;

import java.awt.geom.Point2D;
import java.util.ArrayList;


/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: IGeometriesFactory.java,v 1.1 2006/08/10 12:00:49 jorpiell Exp $
 * $Log: IGeometriesFactory.java,v $
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * This interface must be implemented by all the classes that
 * are able to create geometries from a GML file. When the GML
 * reader parses one geometry (in plain text) it uses this class 
 * to create the equivalent object.
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public interface IGeometriesFactory {
	/**
	 * Creates a point in 2D	 
	 * @param x 
	 * X coordinate
	 * @param y
	 * Y coordinate
	 * @return 
	 * Geometry that represents a point 2D
	 */
	public Object createPoint2D(double x, double y);		

	/**
	 * Creates a point in 2D	 
	 * @param point2D 
	 * Point in 2D
	 * @return 
	 * Geometry that represents a point in 2D
	 */	
	public Object createPoint2D(Point2D point2D);
	
	/**
	 * Creates a multi point in 2D	 
	 * @param x
	 * Array of points with the X coordinate
	 * @param y
	 * Array of points with the Y coordinate
	 * Point in 2D
	 * @return 
	 * Geometry that represents a multi point in 2D
	 */	
	public Object createMultipoint2D(double[] x, double[] y);
	
	/**
	 * Creates a geometry using a GML string
	 * @param gmlString
	 * GML geometry
	 * @return
	 * Geometry represented by the GML string
	 */
	public Object createGeometry(String gmlString);
	
	/**
	 * Create a simple feature
	 * @param element
	 * Element name
	 * @param geometry
	 * Geoemtry 
	 * @param params
	 * Params names
	 * @param values
	 * Params values
	 * @return
	 */
	public Object createSimpleFeature(String element,
			Object geometry,
			ArrayList params,
			ArrayList values);
	
}
