package org.gvsig.remoteClient.gml.schemas;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import org.gvsig.remoteClient.gml.GMLTags;
import org.gvsig.remoteClient.gml.factories.XMLElementsFactory;
import org.gvsig.remoteClient.gml.factories.XMLTypesFactory;
import org.gvsig.remoteClient.utils.CapabilitiesTags;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: XMLSchemaParser.java,v 1.1.2.3 2006/11/17 11:38:03 ppiqueras Exp $
 * $Log: XMLSchemaParser.java,v $
 * Revision 1.1.2.3  2006/11/17 11:38:03  ppiqueras
 * Corregidos bugs y añadida nueva funcionalidad. Del HEAD.
 *
 * Revision 1.4  2006/10/10 12:52:28  jorpiell
 * Soporte para features complejas.
 *
 * Revision 1.3  2006/10/02 08:33:49  jorpiell
 * Cambios del 10 copiados al head
 *
 * Revision 1.1.2.1  2006/09/19 12:23:15  jorpiell
 * Ya no se depende de geotools
 *
 * Revision 1.2  2006/09/18 12:08:55  jorpiell
 * Se han hecho algunas modificaciones que necesitaba el WFS
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 * Revision 1.1  2006/05/16 14:12:56  jorpiell
 * Aadido el parseador de Esquemas
 *
 *
 */
/**
 * Thas class is used to parse a schema XSD
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class XMLSchemaParser extends KXmlParser {
	private String schema = "";
	private String encoding = "UTF-8";	
	private String nameSpace = "";
	private Hashtable attributes = null;
	
	public XMLSchemaParser(){
		super();
		attributes = new Hashtable();
	}
	
	public XMLSchemaParser(String schema){
		super();
		this.schema = schema;
		attributes = new Hashtable();
	}
	
	/**
	 * It gets the schema from a tag. The schema is separated
	 * of the tag name by ":".
	 * @param tag
	 */
	public void setSchemaFromMainTag(String tag){
		int pos = tag.indexOf(":");
		if (pos > 0){
			this.schema = tag.substring(0,pos);
		}else{
			this.schema = "";
		}
	}	

	/**
	 * @return Returns the schema.
	 */
	public String getSchema() {
		return schema;
	}

	/**
	 * @param schema The schema to set.
	 */
	public void setSchema(String schema) {
		this.schema = schema;
	}	
	
	/**
	 * Returns a SCHEMA:TAG
	 * @param tag
	 * @return SCHEMA:TAG
	 */
	private String getTag(String tag){
		if (tag == null){
			return null;
		}
		if ((schema == null) || (schema.equals(""))){
			return tag;
		}
		return schema + ":" + tag;
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.xmlpull.v1.XmlPullParser#require(int, java.lang.String, java.lang.String)
	 */
	public void require(int type, String namespace, String name)
		throws XmlPullParserException, IOException{
		super.require(type,namespace,getTag(name));
	}
	
	/*
	 *  (non-Javadoc)
	 * @see org.xmlpull.v1.XmlPullParser#getName()
	 */
	public String getName(){
		try{
		String name = super.getName();
		if ((schema != null) || (!(schema.equals("")))){
			return name.substring(name.indexOf(":") + 1,name.length());
		}
		return name;
		}catch (NullPointerException e){
			return "";
		}
	}
	
	public void parse(File f,String nameSpace) {
		this.nameSpace = nameSpace;
		FileReader reader = null;       
		try
		{
			reader = new FileReader(f);
			BufferedReader br = new BufferedReader(reader);
			char[] buffer = new char[100];
			br.read(buffer);
			StringBuffer st = new StringBuffer(new String(buffer));
			String searchText = "encoding=\"";
			int index = st.indexOf(searchText);
			if (index>-1) {
				st.delete(0, index+searchText.length());
				encoding = st.substring(0, st.indexOf("\""));
			}
		} catch (FileNotFoundException ex) {
			ex.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
		int tag;
		
		try
		{
			setInput(new FileInputStream(f), encoding);        
			nextTag();
			
			if ( getEventType() != KXmlParser.END_DOCUMENT ) 
			{     
				setSchemaFromMainTag(getName());
				     
				require(KXmlParser.START_TAG, null, CapabilitiesTags.WFS_SCHEMAROOT); 
				tag = nextTag();
				while(tag != KXmlParser.END_DOCUMENT)
				{
					switch(tag)
					{
					case KXmlParser.START_TAG:
						if (getName().compareTo(CapabilitiesTags.COMPLEXTYPE)==0){							
							for (int i=0 ; i<getAttributeCount() ; i++){
					    		if (getAttributeName(i).compareTo(GMLTags.GML_NAME) == 0){
					    			XMLComplexType complexType = XMLTypesFactory.addComplexType(nameSpace,getAttributeValue(i));
					    			parseComplexType(complexType);	
					    			attributes.put(complexType.getName(),complexType);
					    		}
					    		
							}
						} else if (getName().compareTo(CapabilitiesTags.ELEMENT)==0){							
							XMLElement entity = XMLElementsFactory.addType(this);
							try{
								attributes.put(entity.getName(),entity.getEntityType());
							}catch(NullPointerException e){
								//Type not defined
							}
							
						}
						break;
					case KXmlParser.END_TAG:                            
						break;
					case KXmlParser.TEXT:
						if (getName()!=null)
							System.out.println("[TEXT]["+getText().trim()+"]");                         
						break;
					}
					tag = next();
				}
				require(KXmlParser.END_DOCUMENT, null, null);                
			}
		}
		catch(XmlPullParserException parser_ex){
			System.out.println(parser_ex.getMessage());
			parser_ex.printStackTrace();
			return;
		}
		catch (IOException ioe) {           
			ioe.printStackTrace();
			return;
		}		
	}
	
	private void parseComplexType(XMLComplexType complexType) throws IOException, XmlPullParserException{   
		int currentTag;
		boolean end = false;		
		
		require(KXmlParser.START_TAG, null, CapabilitiesTags.COMPLEXTYPE);
		currentTag = next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (getName().compareTo(CapabilitiesTags.COMPLEXCONTENT)==0){
					parseComplexContent(complexType); 
				}else if(getName().compareTo(CapabilitiesTags.SEQUENCE)==0){
					parseSequence(complexType);
				}
				break;
			case KXmlParser.END_TAG:
				if (getName().compareTo(CapabilitiesTags.COMPLEXTYPE) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = next();
			}	
		}		
	}
	
	private void parseComplexContent(XMLComplexType complexType) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		require(KXmlParser.START_TAG, null, CapabilitiesTags.COMPLEXCONTENT);
		currentTag = next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (getName().compareTo(CapabilitiesTags.EXTENSION )==0)
				{
					parseExtension(complexType); 
				}   
				break;
			case KXmlParser.END_TAG:
				if (getName().compareTo(CapabilitiesTags.COMPLEXCONTENT) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = next();
			}	
		}
	}
	
	private void parseExtension(XMLComplexType complexType) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		require(KXmlParser.START_TAG, null, CapabilitiesTags.EXTENSION);
		currentTag = next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (getName().compareTo(CapabilitiesTags.SEQUENCE)==0)
				{
					parseSequence(complexType); 
				}   
				break;
			case KXmlParser.END_TAG:
				if (getName().compareTo(CapabilitiesTags.EXTENSION) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = next();
			}			
		}
	}
	
	private void parseSequence(XMLComplexType complexType) throws IOException, XmlPullParserException
	{   
		int currentTag;
		boolean end = false;
		
		require(KXmlParser.START_TAG, null, CapabilitiesTags.SEQUENCE);
		currentTag = next();
		
		while (!end) 
		{
			switch(currentTag)
			{
			case KXmlParser.START_TAG:
				if (getName().compareTo(CapabilitiesTags.ELEMENT)==0){
					XMLElement element = new XMLElement(this);
					if (element != null){
						complexType.addSubtypes(element);
					}
				}else if(getName().compareTo(CapabilitiesTags.COMPLEXTYPE)==0){
					for (int i=0 ; i<getAttributeCount() ; i++){
			    		if (getAttributeName(i).compareTo(GMLTags.GML_NAME) == 0){
			    			XMLComplexType complexTypeChild = XMLTypesFactory.addComplexType(nameSpace,getAttributeValue(i));
			    			parseComplexType(complexTypeChild);					    			
			    		}			    		
					}
				}
				break;
			case KXmlParser.END_TAG:
				if (getName().compareTo(CapabilitiesTags.SEQUENCE) == 0)
					end = true;
				break;
			case KXmlParser.TEXT:                   
				break;
			}
			if (!end){
				currentTag = next();
			}
		}		
	}

	/**
	 * @return Returns the attributes.
	 */
	public Hashtable getAttributes() {
		return attributes;
	}
	
	public Vector getAttributesList(){
		Vector vector = new Vector();
		Set keys = attributes.keySet();
		Iterator it = keys.iterator();
		while(it.hasNext()){
			vector.add(attributes.get((String)it.next()));
		}
		return vector;
	}

	
}
