package org.gvsig.remoteClient.gml.v2;

import java.io.IOException;

import org.gvsig.remoteClient.gml.GMLException;
import org.gvsig.remoteClient.gml.GMLTags;
import org.gvsig.remoteClient.gml.IGMLFeaturesIterator;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.gvsig.remoteClient.gml.utils.GMLUtilsParser;
import org.gvsig.remoteClient.utils.CapabilitiesTags;
import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLFeaturesParser_v2.java,v 1.2.2.2 2006/11/17 11:38:03 ppiqueras Exp $
 * $Log: GMLFeaturesParser_v2.java,v $
 * Revision 1.2.2.2  2006/11/17 11:38:03  ppiqueras
 * Corregidos bugs y añadida nueva funcionalidad. Del HEAD.
 *
 * Revision 1.4  2006/10/11 16:19:50  jorpiell
 * No se podan cargar GMLs con 0 geometras
 *
 * Revision 1.3  2006/10/10 12:52:28  jorpiell
 * Soporte para features complejas.
 *
 * Revision 1.2  2006/08/10 12:37:05  jorpiell
 * Aadido el tag description al GML parser
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 */
public class GMLFeaturesParser_v2 extends org.gvsig.remoteClient.gml.GMLFeaturesParser {
	
	public GMLFeaturesParser_v2(IGeometriesFactory factory, XMLSchemaParser parser) throws GMLException {
		super(factory, parser);		
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.gml.GMLParser#getFeaturesReader()
	 */
	public IGMLFeaturesIterator getFeaturesReader() throws GMLException {
		return new GMLFeaturesIterator_v2(getParser(),getFactory());
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.remoteClient.gml.GMLParser#initializeHeader()
	 */
	public void initializeHeader() throws GMLException {
		getParser().setSchema("gml");		
		int currentTag;		
		boolean startFeatures = false;
		try {
			currentTag = getParser().next();		
			while(!startFeatures){						
				switch(currentTag){
				case KXmlParser.START_TAG:
					if (getParser().getName().compareTo(GMLTags.GML_NAME)==0){
						
					}else if (getParser().getName().compareTo(GMLTags.GML_DESCRIPTION)==0){
						
					}else if (getParser().getName().compareTo(GMLTags.GML_BOUNDEDBY)==0){
						setExtent(GMLUtilsParser.parseBoundedBy(getParser()));
					}else{
						startFeatures = true;
					}
					break;
				case KXmlParser.END_TAG:		
					if (getParser().getName().compareTo(CapabilitiesTags.WFS_FEATUERE_COLLECTION) == 0){
						startFeatures = true;
					}
					break;
				case KXmlParser.TEXT:                   
					break;
				}
				if (!startFeatures){
					currentTag = getParser().next();
				}
			}
		} catch (XmlPullParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_PARSE);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			throw new GMLException(GMLException.EXC_READ_FILE);
		} 		
	}

	
}
